@@ -51,6 +51,7 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+#include <sys/wait.h>
 
 #include "defaults.h"
 #include "mount_davfs.h"
@@ -167,29 +168,6 @@ init_coda(int *dev, dav_run_msgloop_fn *msg_loop, void **mdata)
         ++minor;
     }
 
-    if (*dev <= 0 && system("/sbin/modprobe coda &>/dev/null") == 0) {
-        minor = 0;
-        while (*dev <= 0 && minor < MAX_CODADEVS) {
-            char *path;
-            if (asprintf(&path, "%s/%s%i",
-                         DAV_DEV_DIR, CODA_DEV_NAME, minor) < 0)
-                abort();
-            *dev = open(path, O_RDWR | O_NONBLOCK);
-            if (*dev <= 0) {
-                if (mknod(path, S_IFCHR, makedev(CODA_MAJOR, minor)) == 0) {
-                    if (chown(path, 0, 0) == 0
-                            && chmod(path, S_IRUSR | S_IWUSR) == 0) {
-                        *dev = open(path, O_RDWR | O_NONBLOCK);
-                    } else {
-                        remove(path);
-                    }
-                }
-            }
-            free(path);
-            ++minor;
-        }
-    }
-
     if (*dev <= 0) {
         error(0, 0, _("no free coda device to mount"));
         return -1;
@@ -225,17 +203,31 @@ init_fuse(int *dev, dav_run_msgloop_fn *msg_loop, void **mdata,
             abort();
 
     *dev = open(path, O_RDWR | O_NONBLOCK);
-    if (*dev <= 0 && system("/sbin/modprobe fuse &>/dev/null") == 0) {
-        *dev = open(path, O_RDWR | O_NONBLOCK);
-    }
+
     if (*dev <= 0) {
-        if (mknod(path, S_IFCHR, makedev(FUSE_MAJOR, FUSE_MINOR)) == 0) {
-             if (chown(path, 0, 0) == 0
-                    && chmod(path, S_IRUSR | S_IWUSR) == 0) {
-                *dev = open(path, O_RDWR | O_NONBLOCK);
-            } else {
-                remove(path);
-            }
+        error(0, 0, _("loading kernel module fuse"));
+        int ret;
+        pid_t pid = fork();
+        if (pid == 0) {
+            execl("/sbin/modprobe", "modprobe", "fuse", NULL);
+            _exit(EXIT_FAILURE);
+        } else if (pid < 0) {
+            ret = -1;
+        } else {
+            if (waitpid(pid, &ret, 0) != pid)
+                ret = -1;
+        }
+
+        if (ret) {
+            error(0, 0, _("loading kernel module fuse failed"));
+        } else {
+            *dev = open(path, O_RDWR | O_NONBLOCK);
+        }
+
+        if (*dev <= 0) {
+            error(0, 0, _("waiting for /dev/fuse to be created"));
+            sleep(2); 
+            *dev = open(path, O_RDWR | O_NONBLOCK);
         }
     }
 
