@@ -31,22 +31,22 @@
    Some data will be copied into global or local variables to be available in
    daemon mode. The rest will be freed when forking into daemon mode. */
 typedef struct {
+    char *cmdline;
     char *dav_user;           /* System config file */
     char *dav_group;          /* System config file */
     char *conf;               /* Command line */
     /* Mount options */
     int user;                 /* Command line */
-    int users;                 /* Command line */
+    int users;                /* Command line */
     int netdev;               /* Command line */
+    int grpid;                /* Command line */
     unsigned long int mopts;  /* Command line */
-    char *add_mopts;
     char *kernel_fs;          /* User config file, system config file */
+    int use_utab;
     size_t buf_size;          /* User config file, system config file */
     /* File mode */
     uid_t uid;                /* Command line */
     gid_t gid;                /* Command line */
-    mode_t dir_umask;
-    mode_t file_umask;
     mode_t dir_mode;          /* Command line */
     mode_t file_mode;         /* Command line */
     /* WebDAV-resource */
@@ -54,7 +54,8 @@ typedef struct {
     char *host;               /* Command line */
     int port;                 /* Command line */
     char *path;               /* Command line */
-    char *servercert;         /* User config file, system config file */
+    char *trust_ca_cert;      /* User config file, system config file */
+    char *trust_server_cert;  /* User config file, system config file */
     char *secrets;            /* User config file */
     char *username;           /* User secrets file, system secrets file */
     char *cl_username;        /* Command line */
@@ -74,9 +75,11 @@ typedef struct {
     int expect100;            /* User config file, system config file */
     int if_match_bug;         /* User config file, system config file */
     int drop_weak_etags;      /* User config file, system config file */
-    int allow_cookie;         /* User config file, system config file */
+    int n_cookies;            /* User config file, system config file */
     int precheck;             /* User config file, system config file */
     int ignore_dav_header;    /* User config file, system config file */
+    int use_compression;      /* User config file, system config file */
+    int follow_redirect;      /* User config file, system config file */
     time_t connect_timeout;   /* User config file, system config file */
     time_t read_timeout;      /* User config file, system config file */
     time_t retry;             /* User config file, system config file */
@@ -94,6 +97,7 @@ typedef struct {
     time_t file_refresh;      /* User config file, system config file */
     int delay_upload;         /* User config file, system config file */
     int gui_optimize;         /* User config file, system config file */
+    int minimize_mem;         /* User config file, system config file */
     /* Debugging */
     int debug;                /* User config file, system config file */
     int neon_debug;           /* User config file, system config file */
