@@ -25,7 +25,9 @@
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#ifdef HAVE_ICONV_H
 #include <iconv.h>
+#endif
 #ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
 #endif
@@ -55,9 +57,11 @@
 #include <ne_alloc.h>
 #include <ne_auth.h>
 #include <ne_basic.h>
+#include <ne_compress.h>
 #include <ne_dates.h>
 #include <ne_locks.h>
 #include <ne_props.h>
+#include <ne_redirect.h>
 #include <ne_request.h>
 #include <ne_session.h>
 #include <ne_socket.h>
@@ -108,7 +112,6 @@ typedef struct {
 enum {
     ETAG = 0,
     LENGTH,
-    CREATION,
     MODIFIED,
     TYPE,
     EXECUTE,
@@ -118,7 +121,6 @@ enum {
 static const ne_propname prop_names[] = {
     [ETAG] = {"DAV:", "getetag"},
     [LENGTH] = {"DAV:", "getcontentlength"},
-    [CREATION] ={"DAV:", "creationdate"},
     [MODIFIED] = {"DAV:", "getlastmodified"},
     [TYPE] = {"DAV:", "resourcetype"},
     [EXECUTE] = {"http://apache.org/dav/props/", "executable"},
@@ -128,7 +130,6 @@ static const ne_propname prop_names[] = {
 static const ne_propname anonymous_prop_names[] = {
     [ETAG] = {NULL, "getetag"},
     [LENGTH] = {NULL, "getcontentlength"},
-    [CREATION] ={NULL, "creationdate"},
     [MODIFIED] = {NULL, "getlastmodified"},
     [TYPE] = {NULL, "resourcetype"},
     [EXECUTE] = {NULL, "executable"},
@@ -160,6 +161,12 @@ static char *none_match_header = "If-None-Match: *\r\n";
    Will be set by dav_init_webdav(). */
 static ne_session *session;
 
+/* Timeout for requests. Set by dav_init_webdav(). */
+static int read_timeout;
+
+/* Timeout for creating a connection. Set by dav_init_webdav(). */
+static int connect_timeout;
+
 /* Lock store, lock owner and lock timeout for session.
    Will be set by dav_init_webdav(). */
 static ne_lock_store *locks;
@@ -172,6 +179,9 @@ static char *password;
 static char *p_username;
 static char *p_password;
 
+/* If this is not NULL the server must present exactly this certificate. */
+static ne_ssl_certificate *server_cert;
+
 /* Whether to send expect 100-continue header in PUT requests. */
 static int use_expect100;
 
@@ -191,6 +201,9 @@ static int precheck;
 /* Ignore the information in the DAV-header because it is wrong. */
 static int ignore_dav_header;
 
+/* Use "Content-Encoding: gzip" for GET requests. */
+static int use_compression;
+
 /* Will be set to 1 when dav_init_connection() succeeded. */
 static int initialized;
 
@@ -199,6 +212,7 @@ static int initialized;
    Needed by  ssl_verify() which may be called at any time. */
 static int have_terminal;
 
+#ifdef HAVE_ICONV_H
 /* Handle to convert character encoding from utf-8 to LC_CTYPE.
    If NULL no conversion is done. */
 static iconv_t from_utf_8;
@@ -214,6 +228,7 @@ static iconv_t from_server_enc;
 /* Handle to convert from LC_CTYPE to character encoding of path names.
    If NULL no conversion is done. */
 static iconv_t to_server_enc;
+#endif
 
 /* A GNU custom stream, used to redirect neon debug messages to syslog. */
 static FILE *log_stream;
@@ -221,23 +236,32 @@ static FILE *log_stream;
 /* A user defined header that is added to all requests. */
 char *custom_header;
 
-#if NE_VERSION_MINOR > 25
-/* Session cookie. */
-static char *cookie;
-#endif
+/* Array of max. n_cookies cookie strings. NULL if configuration option
+   allow_cookies is 0. The array must be allocated by dav_init_webdav. */
+static int n_cookies;
+static char **cookie_list;
 
 
 /* Private function prototypes and inline functions */
 /*==================================================*/
 
+#ifdef HAVE_ICONV_H
 static void
 convert(char **s, iconv_t conv);
+#endif
+
+static ne_session *
+create_rd_session(const ne_uri *uri);
 
 static int
-get_error(int ret, const char *method);
+get_error(int ret, const char *method, ne_session *sess);
 
 static int
-get_ne_error(const char *method);
+get_ne_error(const char *method, ne_session *sess);
+
+int
+get_redirect(const ne_uri *uri, const char *cache_path, off_t *size,
+             char **etag, time_t *mtime, int *modified);
 
 static struct ne_lock *
 lock_by_path(const char *path);
@@ -268,19 +292,8 @@ auth(void *userdata, const char *realm, int attempt, char *user, char *pwd);
 static int
 file_reader(void *userdata, const char *block, size_t length);
 
-#if NE_VERSION_MINOR < 26
-
 static void
-lock_result(void *userdata, const struct ne_lock *lock, const char *uri,
-            const ne_status *status);
-
-static void
-prop_result(void *userdata, const char *href, const ne_prop_result_set *set);
-
-static void
-quota_result(void *userdata, const char *href, const ne_prop_result_set *set);
-
-#else /* NE_VERSION_MINOR >= 26 */
+get_cookies(ne_request *req, void *userdata, const ne_status *status);
 
 static void
 lock_result(void *userdata, const struct ne_lock *lock, const ne_uri *uri,
@@ -292,19 +305,12 @@ prop_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set);
 static void
 quota_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set);
 
-#endif /* NE_VERSION_MINOR >= 26 */
-
 static int
 quota_reader(void *userdata, const char *block, size_t length);
 
 static int
 ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert);
 
-#if NE_VERSION_MINOR > 25
-static int
-update_cookie(ne_request *req, void *userdata, const ne_status *status);
-#endif /* NE_VERSION_MINOR > 25 */
-
 
 /* Public functions */
 /*==================*/
@@ -315,6 +321,7 @@ dav_init_webdav(const dav_args *args)
     if (args->neon_debug & ~NE_DBG_HTTPPLAIN)
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "Initializing webdav");
 
+#ifdef HAVE_ICONV_H
     char *lc_charset = nl_langinfo(CODESET);
     if (lc_charset && strcasecmp(lc_charset, "UTF-8") != 0) {
         from_utf_8 = iconv_open(lc_charset, "UTF-8");
@@ -339,6 +346,7 @@ dav_init_webdav(const dav_args *args)
                 to_server_enc = 0;
         }
     }
+#endif /* HAVE_ICONV */
 
     if (ne_sock_init() != 0)
         error(EXIT_FAILURE, errno, _("socket library initialization failed"));
@@ -360,25 +368,23 @@ dav_init_webdav(const dav_args *args)
 
     session = ne_session_create(args->scheme, args->host, args->port);
 
-    ne_set_read_timeout(session, args->read_timeout);
-
-#if NE_VERSION_MINOR > 26
-    ne_set_connect_timeout(session, args->connect_timeout);
-#endif /* NE_VERSION_MINOR > 26 */
+    read_timeout = args->read_timeout;
+    ne_set_read_timeout(session, read_timeout);
+    connect_timeout = args->connect_timeout;
+    ne_set_connect_timeout(session, connect_timeout);
 
     char *useragent = ne_concat(PACKAGE_TARNAME, "/", PACKAGE_VERSION, NULL);
     ne_set_useragent(session, useragent);
     free(useragent);
 
+    if (args->follow_redirect)
+        ne_redirect_register(session);
+
     if (args->username)
         username = ne_strdup(args->username);
     if (args->password)
         password = ne_strdup(args->password);
-#if NE_VERSION_MINOR < 26
-    ne_set_server_auth(session, auth, "server");
-#else /* NE_VERSION_MINOR >= 26 */
     ne_add_server_auth(session, NE_AUTH_ALL, auth, "server");
-#endif /* NE_VERSION_MINOR >= 26 */
 
     if (args->useproxy && args->p_host) {
         ne_session_proxy(session, args->p_host, args->p_port);
@@ -386,27 +392,28 @@ dav_init_webdav(const dav_args *args)
             p_username = ne_strdup(args->p_user);
         if (args->p_passwd)
             p_password = ne_strdup(args->p_passwd);
-#if NE_VERSION_MINOR < 26
-        ne_set_proxy_auth(session, auth, "proxy");
-#else /* NE_VERSION_MINOR >= 26 */
         ne_add_proxy_auth(session, NE_AUTH_ALL, auth, "proxy");
-#endif /* NE_VERSION_MINOR >= 26 */
     }
 
+
     if (strcmp(args->scheme, "https") == 0) {
         if (!ne_has_support(NE_FEATURE_SSL))
             error(EXIT_FAILURE, 0, _("neon library does not support TLS/SSL"));
-        ne_ssl_set_verify(session, ssl_verify, NULL);
-        ne_ssl_trust_default_ca(session);
 
-        if (args->servercert) {
-            ne_ssl_certificate *server_cert
-                    = ne_ssl_cert_read(args->servercert);
+        ne_ssl_set_verify(session, ssl_verify, NULL);
+        if (args->trust_server_cert) {
+            server_cert = ne_ssl_cert_read(args->trust_server_cert);
             if (!server_cert)
                 error(EXIT_FAILURE, 0, _("can't read server certificate %s"),
-                      args->servercert);
-            ne_ssl_trust_cert(session, server_cert);
-            ne_ssl_cert_free(server_cert);
+                      args->trust_server_cert);
+        } else if (args->trust_ca_cert) {
+            ne_ssl_certificate *ca_cert = ne_ssl_cert_read(args->trust_ca_cert);
+            if (!ca_cert)
+                error(EXIT_FAILURE, 0, _("can't read server certificate %s"),
+                      args->trust_ca_cert);
+            ne_ssl_trust_cert(session, ca_cert);
+        } else {
+            ne_ssl_trust_default_ca(session);
         }
 
         if (args->clicert) {
@@ -464,19 +471,23 @@ dav_init_webdav(const dav_args *args)
 
     if (args->header) {
         custom_header = ne_strdup(args->header);
-        ne_hook_pre_send(session, add_header, custom_header);
     }
 
-#if NE_VERSION_MINOR > 25
-    if (args->allow_cookie)
-        ne_hook_post_send(session, update_cookie, NULL);
-#endif /* NE_VERSION_MINOR > 25 */
+    if (args->n_cookies) {
+        n_cookies = args->n_cookies;
+        cookie_list = (char **) ne_calloc(n_cookies * sizeof(char *));
+        ne_hook_post_headers(session, get_cookies, NULL);
+    }
+
+    if (custom_header || cookie_list)
+        ne_hook_pre_send(session, add_header, NULL);
 
     use_expect100 = args->expect100;
     has_if_match_bug = args->if_match_bug;
     drop_weak_etags = args->drop_weak_etags;
     precheck = args->precheck;
     ignore_dav_header = args->ignore_dav_header;
+    use_compression = args->use_compression & ne_has_support(NE_FEATURE_ZLIB);
 }
 
 
@@ -508,7 +519,7 @@ dav_init_connection(const char *path)
             locks = NULL;
         }
     } else {
-        ret = get_error(ret, "OPTIONS");
+        ret = get_error(ret, "OPTIONS", session);
     }
 
     free(spath);
@@ -528,7 +539,7 @@ dav_close_webdav(void)
         while (lock) {
             if (session) {
                 int ret = ne_unlock(session, lock);
-                get_error(ret, "UNLOCK");
+                get_error(ret, "UNLOCK", session);
             }
             lock = ne_lockstore_next(locks);
         }
@@ -544,8 +555,10 @@ char *
 dav_conv_from_utf_8(const char *s)
 {
     char *new = ne_strdup(s);
+#ifdef HAVE_ICONV
     if (from_utf_8)
         convert(&new, from_utf_8);
+#endif
     return new;
 }
 
@@ -554,8 +567,10 @@ char *
 dav_conv_to_utf_8(const char *s)
 {
     char *new = ne_strdup(s);
+#ifdef HAVE_ICONV
     if (to_utf_8)
         convert(&new, to_utf_8);
+#endif
     return new;
 }
 
@@ -564,8 +579,10 @@ char *
 dav_conv_from_server_enc(const char *s)
 {
     char *new = ne_strdup(s);
+#ifdef HAVE_ICONV
     if (from_server_enc)
         convert(&new, from_server_enc);
+#endif
     return new;
 }
 
@@ -574,8 +591,10 @@ char *
 dav_conv_to_server_enc(const char *s)
 {
     char *new = ne_strdup(s);
+#ifdef HAVE_ICONV
     if (to_server_enc)
         convert(&new, to_server_enc);
+#endif
     return new;
 }
 
@@ -592,14 +611,14 @@ dav_delete(const char *path, time_t *expire)
     struct ne_lock *lock = NULL;
     char *spath = ne_path_escape(path);
     ret = ne_delete(session, spath);
-    ret = get_error(ret, "DELETE");
+    ret = get_error(ret, "DELETE", session);
 
     if ((ret == EACCES || ret == ENOENT) && locks) {
         lock_discover(spath, expire);
         lock = lock_by_path(spath);
         if (lock && ret == EACCES) {
             ret = ne_delete(session, spath);
-            ret = get_error(ret, "DELETE");
+            ret = get_error(ret, "DELETE", session);
         } else if (lock) {
             ne_unlock(session, lock);
             ret = 0;
@@ -630,7 +649,7 @@ dav_delete_dir(const char *path)
 
     char *spath = ne_path_escape(path);
     ret = ne_delete(session, spath);
-    ret = get_error(ret, "DELETE");
+    ret = get_error(ret, "DELETE", session);
 
     free(spath);
     return ret;
@@ -666,7 +685,7 @@ dav_get_collection(const char *path, dav_props **props)
     char *spath = ne_path_escape(path);
     ne_propfind_handler *ph = ne_propfind_create(session, spath, NE_DEPTH_ONE);
     ret = ne_propfind_named(ph, prop_names, prop_result, &ctx);
-    ret = get_error(ret, "PROPFIND");
+    ret = get_error(ret, "PROPFIND", session);
     ne_propfind_destroy(ph);
     free(spath);
 
@@ -692,7 +711,7 @@ dav_get_webdav_error()
 
 int
 dav_get_file(const char *path, const char *cache_path, off_t *size,
-             char **etag, time_t *mtime, char **mime, int *modified)
+             char **etag, time_t *mtime, int *modified)
 {
     int ret;
     if (!initialized) {
@@ -716,10 +735,48 @@ dav_get_file(const char *path, const char *cache_path, off_t *size,
         ne_add_request_header(req, "If-Modified-Since", mod_time);
     }
 
-    ne_add_response_body_reader(req, ne_accept_2xx, file_reader, &ctx);
+    ne_decompress *dc_state = NULL;
+    if (use_compression) {
+        dc_state = ne_decompress_reader(req, ne_accept_2xx, file_reader, &ctx);
+    } else {
+        ne_add_response_body_reader(req, ne_accept_2xx, file_reader, &ctx);
+    }
 
     ret = ne_request_dispatch(req);
-    ret = get_error(ret, "GET");
+
+    if (use_compression)
+        ne_decompress_destroy(dc_state);
+
+    ne_session *rd_sess = NULL;
+    const ne_uri *rd_uri = NULL;
+    if (ret == NE_REDIRECT)
+        rd_uri = ne_redirect_location(session);
+
+    if (rd_uri) {
+        rd_sess = create_rd_session(rd_uri);
+
+        ne_request_destroy(req);
+        req = ne_request_create(rd_sess, "GET", rd_uri->path);
+
+        if (etag && *etag)
+            ne_add_request_header(req, "If-None-Match", *etag);
+
+        if (use_compression) {
+            dc_state = ne_decompress_reader(req, ne_accept_2xx, file_reader,
+                                            &ctx);
+        } else {
+            ne_add_response_body_reader(req, ne_accept_2xx, file_reader, &ctx);
+        }
+
+        ret = ne_request_dispatch(req);
+
+        if (use_compression)
+            ne_decompress_destroy(dc_state);
+        ret = get_error(ret, "GET", rd_sess);
+    } else {
+        ret = get_error(ret, "GET", session);
+    }
+
     if (ctx.error)
         ret = ctx.error;
     const ne_status *status = ne_get_status(req);
@@ -746,15 +803,11 @@ dav_get_file(const char *path, const char *cache_path, off_t *size,
             *etag = normalize_etag(ne_get_response_header(req, "ETag"));
         }
 
-        value = ne_get_response_header(req, "Content-Type");
-        if (mime && value) {
-            if (*mime)
-                free(*mime);
-            *mime = ne_strdup(value);
-        }
     }
 
     ne_request_destroy(req);
+    if (rd_sess)
+        ne_session_destroy(rd_sess);
     if (ctx.fd > 0)
         close(ctx.fd);
     if (mod_time)
@@ -765,8 +818,7 @@ dav_get_file(const char *path, const char *cache_path, off_t *size,
 
 
 int
-dav_head(const char *path, char **etag, time_t *mtime, off_t *length,
-         char **mime)
+dav_head(const char *path, char **etag, time_t *mtime, off_t *length)
 {
     int ret;
     if (!initialized) {
@@ -777,7 +829,7 @@ dav_head(const char *path, char **etag, time_t *mtime, off_t *length,
     char *spath = ne_path_escape(path);
     ne_request *req = ne_request_create(session, "HEAD", spath);
     ret = ne_request_dispatch(req);
-    ret = get_error(ret, "HEAD");
+    ret = get_error(ret, "HEAD", session);
 
     const char *value = ne_get_response_header(req, "ETag");
     if (!ret && etag && value) {
@@ -796,13 +848,6 @@ dav_head(const char *path, char **etag, time_t *mtime, off_t *length,
     if (!ret && length && value)
         *length = strtol(value, NULL, 10);
 
-    value = ne_get_response_header(req, "Content-Type");
-    if (!ret && mime && value) {
-        if (*mime)
-            free(*mime);
-        *mime = ne_strdup(value);
-    }
-
     ne_request_destroy(req);
     free(spath);
     return ret;
@@ -823,12 +868,8 @@ dav_lock(const char *path, time_t *expire, int *exists)
         return 0;
     }
 
-#if NE_VERSION_MINOR > 25
     if (precheck && has_if_match_bug && !*exists) {
-#else /* NE_VERSION_MINOR == 25 */
-    if (precheck && !*exists) {
-#endif /* NE_VERSION_MINOR == 25 */
-        if (dav_head(path, NULL, NULL, NULL, NULL) == 0) {
+        if (dav_head(path, NULL, NULL, NULL) == 0) {
             return EEXIST;
         }
     }
@@ -849,19 +890,15 @@ dav_lock(const char *path, time_t *expire, int *exists)
     lock->owner = ne_strdup(owner);
     lock->timeout = lock_timeout;
 
-#if NE_VERSION_MINOR > 25
     if (!has_if_match_bug && !*exists)
         ne_hook_pre_send(session, add_header, none_match_header);
-#endif /* NE_VERSION_MINOR > 25 */
     ret = ne_lock(session, lock);
-    ret = get_error(ret, "LOCK");
-#if NE_VERSION_MINOR > 25
+    ret = get_error(ret, "LOCK", session);
     if (!has_if_match_bug && !*exists) {
         ne_unhook_pre_send(session, add_header, none_match_header);
         if (ret && strtol(ne_get_error(session), NULL, 10) == 412)
             ret = EEXIST;
     }
-#endif /* NE_VERSION_MINOR > 25 */
 
     if (!ret) {
         ne_lockstore_add(locks, lock);
@@ -917,7 +954,7 @@ dav_make_collection(const char *path)
 
     char *spath = ne_path_escape(path);
     ret = ne_mkcol(session, spath);
-    ret = get_error(ret, "MKCOL");
+    ret = get_error(ret, "MKCOL", session);
 
     free(spath);
     return ret;
@@ -936,7 +973,7 @@ dav_move(const char *src, const char *dst)
     char *spath = ne_path_escape(src);
     char *dst_path = ne_path_escape(dst);
     ret = ne_move(session, 1, spath, dst_path); 
-    ret = get_error(ret, "MOVE");
+    ret = get_error(ret, "MOVE", session);
 
     if (!ret && locks) {
         struct ne_lock *lock = lock_by_path(spath);
@@ -954,7 +991,7 @@ dav_move(const char *src, const char *dst)
 
 int
 dav_put(const char *path, const char *cache_path, int *exists, time_t *expire,
-        char **etag, time_t *mtime, char **mime, int execute)
+        char **etag, time_t *mtime, int execute)
 {
     int ret = 0;
     if (!initialized) {
@@ -968,7 +1005,7 @@ dav_put(const char *path, const char *cache_path, int *exists, time_t *expire,
         char *r_etag = NULL;
         time_t r_mtime = 0;
         off_t r_length = 0;
-        ret = dav_head(path, &r_etag, &r_mtime, &r_length, NULL);
+        ret = dav_head(path, &r_etag, &r_mtime, &r_length);
         if (!ret) {
             if (!*exists && r_length) {
                 ret = EEXIST;
@@ -1012,21 +1049,13 @@ dav_put(const char *path, const char *cache_path, int *exists, time_t *expire,
     }
 
     if (use_expect100)
-#if NE_VERSION_MINOR == 25
-        ne_set_request_expect100(req, 1);
-#else /* NE_VERSION_MINOR > 25 */
         ne_set_request_flag(req, NE_REQFLAG_EXPECT100, 1);
-#endif /* NE_VERSION_MINOR > 25 */
 
     ne_lock_using_resource(req, spath, 0);
-#if _FILE_OFFSET_BITS == 64 && NE_VERSION_MINOR < 27
-    ne_set_request_body_fd64(req, fd, 0, st.st_size);
-#else /* _FILE_OFFSET_BITS != 64 || NE_VERSION_MINOR >= 27 */
     ne_set_request_body_fd(req, fd, 0, st.st_size);
-#endif /* _FILE_OFFSET_BITS != 64 || NE_VERSION_MINOR >= 27 */
 
     ret = ne_request_dispatch(req);
-    ret = get_error(ret, "PUT");
+    ret = get_error(ret, "PUT", session);
 
     if (ret == EACCES && lock_discover(spath, expire) == 0) {
 
@@ -1043,21 +1072,13 @@ dav_put(const char *path, const char *cache_path, int *exists, time_t *expire,
         }
 
         if (use_expect100)
-#if NE_VERSION_MINOR == 25
-            ne_set_request_expect100(req, 1);
-#else /* NE_VERSION_MINOR > 25 */
             ne_set_request_flag(req, NE_REQFLAG_EXPECT100, 1);
-#endif /* NE_VERSION_MINOR > 25 */
 
         ne_lock_using_resource(req, spath, 0);
-#if _FILE_OFFSET_BITS == 64  && NE_VERSION_MINOR < 27
-        ne_set_request_body_fd64(req, fd, 0, st.st_size);
-#else /* _FILE_OFFSET_BITS != 64 || NE_VERSION_MINOR >= 27 */
         ne_set_request_body_fd(req, fd, 0, st.st_size);
-#endif /* _FILE_OFFSET_BITS != 64 || NE_VERSION_MINOR >= 27 */
 
         ret = ne_request_dispatch(req);
-        ret = get_error(ret, "PUT");
+        ret = get_error(ret, "PUT", session);
     }
 
     int need_head = 0;
@@ -1084,14 +1105,6 @@ dav_put(const char *path, const char *cache_path, int *exists, time_t *expire,
             }
         }
 
-        if (mime) {
-            value = ne_get_response_header(req, "Content-Type");
-            if (value) {
-                if (*mime)
-                    free(*mime);
-                *mime = ne_strdup(value);
-            }
-        }
     }
 
     ne_request_destroy(req);
@@ -1102,7 +1115,7 @@ dav_put(const char *path, const char *cache_path, int *exists, time_t *expire,
         if (execute == 1)
             dav_set_execute(path, execute);
         if (need_head)
-            dav_head(path, etag, mtime, NULL, mime);
+            dav_head(path, etag, mtime, NULL);
     }
 
     return ret;
@@ -1132,7 +1145,7 @@ dav_quota(const char *path, off64_t *total, off64_t *used)
         ne_propfind_handler *ph = ne_propfind_create(session, spath,
                                                      NE_DEPTH_ZERO);
         ret = ne_propfind_named(ph, quota_names, quota_result, &ctx);
-        ret = get_error(ret, "PROPFIND");
+        ret = get_error(ret, "PROPFIND", session);
         ne_propfind_destroy(ph);
 
         if (!ret && ctx.error) {
@@ -1147,7 +1160,7 @@ dav_quota(const char *path, off64_t *total, off64_t *used)
         ne_request *req = ne_request_create(session, "USERINFO", spath);
         ne_add_response_body_reader(req, ne_accept_2xx, quota_reader, &ctx);
         ret = ne_request_dispatch(req);
-        ret = get_error(ret, "USERINFO");
+        ret = get_error(ret, "USERINFO", session);
         ne_request_destroy(req);
 
         if (!ret) {
@@ -1189,7 +1202,7 @@ dav_set_execute(const char *path, int set)
 
     char *spath = ne_path_escape(path);
     ret = ne_proppatch(session, spath, &op[0]);
-    ret = get_error(ret, "PROPPATCH");
+    ret = get_error(ret, "PROPPATCH", session);
     free(spath);
 
     return ret;
@@ -1227,7 +1240,7 @@ dav_unlock(const char *path, time_t *expire)
     }
 
     ret = ne_unlock(session, lock);
-    ret = get_error(ret, "UNLOCK");
+    ret = get_error(ret, "UNLOCK", session);
 
     if (!ret || ret == ENOENT || ret == EINVAL) {
         ne_lockstore_remove(locks, lock);
@@ -1244,6 +1257,7 @@ dav_unlock(const char *path, time_t *expire)
 /* Private functions */
 /*===================*/
 
+#ifdef HAVE_ICONV
 static void
 convert(char **s, iconv_t conv)
 {
@@ -1264,6 +1278,62 @@ convert(char **s, iconv_t conv)
 
     free(buf);
 }
+#endif /* HAVE_ICONV */
+
+
+/* Creates a new session.
+   Scheme, host and port are taken from uri.
+   read_timout, connect_timeout, useragent and proxy are the same as for the
+   main sessions. The values are taken from the main session or from global
+   variables.
+   The same call back functions for auth and custom headers as for the main
+   session are registered, using the same credentials as the main session.
+   If the scheme is https the ssl_verify function is registered. It is set up
+   to trust in the default CAs. A server certificate or client certificate
+   configured by the user are not used because it may be a different server.
+   TODO: usage of certificates and credentials should depend on uri->host.
+   Return value : the newly created session. It must be destroyed by the
+                  caller. */
+static ne_session *
+create_rd_session(const ne_uri *uri)
+{
+    if (!uri->scheme || !uri->host || !uri->path)
+        return NULL;
+    if (strcmp(uri->scheme, "https") == 0 && !ne_has_support(NE_FEATURE_SSL))
+        return NULL;
+
+    int port = uri->port;
+    if (port == 0)
+        port = ne_uri_defaultport(uri->scheme);
+
+    ne_session *rd_sess = ne_session_create(uri->scheme, uri->host, port);
+    
+    ne_set_read_timeout(rd_sess, read_timeout);
+    ne_set_connect_timeout(rd_sess, connect_timeout);
+
+    char *useragent = ne_concat(PACKAGE_TARNAME, "/", PACKAGE_VERSION, NULL);
+    ne_set_useragent(session, useragent);
+    free(useragent);
+
+    ne_add_server_auth(rd_sess, NE_AUTH_ALL, auth, "server");
+
+    ne_uri *proxy = (ne_uri *) ne_calloc(sizeof(ne_uri));
+    ne_fill_proxy_uri(rd_sess, proxy);
+    if (proxy->host) {
+        ne_session_proxy(rd_sess, proxy->host, proxy->port);
+        ne_add_proxy_auth(rd_sess, NE_AUTH_ALL, auth, "proxy");
+    }    
+
+    if (strcmp(uri->scheme, "https") == 0) {
+        ne_ssl_set_verify(rd_sess, ssl_verify, NULL);
+        ne_ssl_trust_default_ca(rd_sess);
+    }
+
+    if (custom_header)
+        ne_hook_pre_send(rd_sess, add_header, custom_header);
+
+    return rd_sess;
+}
 
 
 /* Returns a file error code according to ret from the last WebDAV
@@ -1271,15 +1341,16 @@ convert(char **s, iconv_t conv)
    fetched and translated.
    ret    : the error code returned from NEON.
    method : name of the WebDAV method, used for debug messages.
+   sess   : the session to get the HTTP status code from.
    return value : a file error code according to errno.h. */
 static int
-get_error(int ret, const char *method)
+get_error(int ret, const char *method, ne_session *sess)
 {
     int err;
     switch (ret) {
     case NE_OK:
     case NE_ERROR:
-        err = get_ne_error(method);
+        err = get_ne_error(method, sess);
         break;
     case NE_LOOKUP:
         err = EIO;
@@ -1316,11 +1387,12 @@ get_error(int ret, const char *method)
 
 /* Get the error from the session and translates it into a file error code.
    method : name of the WebDAV method, used for debug messages.
+   sess   : the session to get the HTTP status code from.
    return value : a file error code according to errno.h. */
 static int
-get_ne_error(const char *method)
+get_ne_error(const char *method, ne_session *sess)
 {
-    const char *text = ne_get_error(session);
+    const char *text = ne_get_error(sess);
 
     char *tail;
     int err = strtol(text, &tail, 10);
@@ -1423,7 +1495,7 @@ lock_discover(const char *path, time_t *expire)
 
     struct ne_lock *lock = NULL;
     int ret = ne_lock_discover(session, path, lock_result, &lock);
-    ret = get_error(ret, "LOCKDISCOVER");
+    ret = get_error(ret, "LOCKDISCOVER", session);
 
     if (!ret && lock)  {
         *expire = -1;
@@ -1445,7 +1517,7 @@ static void
 lock_refresh(struct ne_lock *lock, time_t *expire)
 {
     int ret = ne_lock_refresh(session, lock);
-    ret = get_error(ret, "LOCKREFRESH");
+    ret = get_error(ret, "LOCKREFRESH", session);
 
     if (!ret) {
         if (lock->timeout <= 0) {
@@ -1549,7 +1621,20 @@ replace_slashes(char **name)
 static void
 add_header(ne_request *req, void *userdata, ne_buffer *header)
 {
-    ne_buffer_zappend(header, (char *) userdata);
+    if (custom_header)
+        ne_buffer_zappend(header, custom_header);
+
+    if (cookie_list && cookie_list[0]) {
+        ne_buffer_zappend(header, "Cookie: ");
+        ne_buffer_zappend(header, cookie_list[0]);
+        int i = 1;
+        while (i < n_cookies && cookie_list[i]) {
+            ne_buffer_zappend(header, ", ");
+            ne_buffer_zappend(header, cookie_list[i]);
+            i++;
+        }
+        ne_buffer_zappend(header, "\r\n");
+    }
 }
 
 
@@ -1628,6 +1713,71 @@ file_reader(void *userdata, const char *block, size_t length)
 }
 
 
+/* A ne_post_header_fn to read cookies from the Set-Cookie header and
+   store them in the global arrray of strings cookie_list. The cookies
+   will be send in the Cookie header of subsequent requests.
+   Only the "name=value" part of the cookie is stored. All attributes
+   are completely ignored.
+   When a cookie with the same name as an already stored cookie, but with
+   a different value is received, it's value is updated if necessary.
+   Only n_cookies cookies will be stored. If the server sends more
+   different cookies these will be ignored.
+   status must be of class 2XX or 3XX, otherwise the cookie is ignored. */
+static void
+get_cookies(ne_request *req, void *userdata, const ne_status *status)
+{
+    if (status->klass != 2 && status->klass != 3)
+        return;
+
+    const char *cookie_hdr = ne_get_response_header(req, "Set-Cookie");
+    if (!cookie_hdr)
+        return;
+
+    const char *next = cookie_hdr;
+    while (next) {
+        const char *start = next;
+        next = strchr(start, ',');
+        const char *end = strchr(start, ';');
+        if (next) {
+            if (!end || end > next)
+                end = next;
+            next++;
+        } else if (!end) {
+            end = start + strlen(start);
+        }
+
+        while (start < end && *start == ' ')
+            start++;
+        while (end > start && *(end - 1) == ' ')
+            end--;
+
+        if ((start + 4) > end || *start == '=' || *(end - 1) == '=')
+            continue;
+
+        char *es = strchr(start, '=');
+        if (!es)
+            continue;
+        size_t nl = es - start;
+        size_t vl = end - es - 1;
+
+        int i = 0;
+        for (i = 0; i < n_cookies; i++) {
+            if (!cookie_list[i]) {
+                cookie_list[i] = ne_strndup(start, end - start);
+                break;
+            }
+            if (strncmp(cookie_list[i], start, nl) == 0) {
+                if (strncmp(cookie_list[i] + nl + 1, es + 1, vl) != 0) {
+                    free(cookie_list[i]);
+                    cookie_list[i] = ne_strndup(start, end - start);
+                }
+                break;
+            }
+        }
+    }
+}
+
+
 /* If the owner of this lock is the same as global variable owner, lock is
    stored in the global lock store locks and a pointer to the lock is
    returned in userdata.
@@ -1636,23 +1786,10 @@ file_reader(void *userdata, const char *block, size_t length)
    lock     : a lock found by ne_lock_discover() on the server.
    uri      : not used.
    status   : not used. */
- 
-#if NE_VERSION_MINOR < 26
-
-static void
-lock_result(void *userdata, const struct ne_lock *lock, const char *uri,
-            const ne_status *status)
-{
-
-#else     /* NE_VERSION_MINOR >= 26 */
-
 static void
 lock_result(void *userdata, const struct ne_lock *lock, const ne_uri *uri,
             const ne_status *status)
 {
-
-#endif    /* NE_VERSION_MINOR >= 26 */
-
     if (!locks || !owner || !userdata || !lock || !lock->owner)
         return;
 
@@ -1682,34 +1819,8 @@ lock_result(void *userdata, const struct ne_lock *lock, const ne_uri *uri,
    that is not a directory or that is less specific.
    userdata : A pointer to a propfind_context structure containing the path of
               the collection and the linked list of properties.
-if NE_VERSION_MINOR < 26
-   href     : Value of the href propertiy returned by the server. It may be
-              the complete URL of the collection or the path only.
-else
    uri      : ne_uri of the resource as returned from the server.
-endif
    set      : Points to the set of properties returned from the server.*/
-
-#if NE_VERSION_MINOR < 26
-
-static void
-prop_result(void *userdata, const char *href, const ne_prop_result_set *set)
-{
-    propfind_context *ctx = (propfind_context *) userdata;
-    if (!ctx || !href || !set)
-        return;
-
-    ne_uri uri;
-    if (ne_uri_parse(href, &uri) != 0 || !uri.path) {
-        ne_uri_free(&uri);
-        return;
-    }
-
-    char *tmp_path = (char *) ne_malloc(strlen(uri.path) + 1);
-    const char *from = uri.path;
-
-#else /* NE_VERSION_MINOR >= 26 */
-
 static void
 prop_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
 {
@@ -1719,9 +1830,6 @@ prop_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
 
     char *tmp_path = (char *) ne_malloc(strlen(uri->path) + 1);
     const char *from = uri->path;
-
-#endif /* NE_VERSION_MINOR >= 26 */
-
     char *to = tmp_path;
     while (*from) {
         while (*from == '/' && *(from + 1) == '/')
@@ -1733,11 +1841,6 @@ prop_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
     result->path = ne_path_unescape(tmp_path);
     free (tmp_path);
 
-#if NE_VERSION_MINOR < 26
-    ne_uri_free(&uri);
-#endif
-
-
     if (!result->path || strlen(result->path) < 1) {
         dav_delete_props(result);
         return;
@@ -1778,8 +1881,10 @@ prop_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
                                   strlen(result->path) - strlen(ctx->path)
                                   - result->is_dir);
         replace_slashes(&result->name);
+#ifdef HAVE_ICONV
         if (from_server_enc)
             convert(&result->name, from_server_enc);
+#endif
     }
 
     data = ne_propset_value(set, &prop_names[ETAG]);
@@ -1797,17 +1902,6 @@ prop_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
          result->size = strtol(data, NULL, 10);
 #endif /* _FILE_OFFSET_BITS != 64 */
 
-    data = ne_propset_value(set, &prop_names[CREATION]);
-    if (!data)
-        data = ne_propset_value(set, &anonymous_prop_names[CREATION]);
-    if (data) {
-        result->ctime = ne_iso8601_parse(data);
-        if (result->ctime == (time_t) -1)
-            result->ctime = ne_httpdate_parse(data);
-        if (result->ctime == (time_t) -1)
-            result->ctime = 0;
-    }
-
     data = ne_propset_value(set, &prop_names[MODIFIED]);
     if (!data)
         data = ne_propset_value(set, &anonymous_prop_names[MODIFIED]);
@@ -1839,7 +1933,7 @@ quota_reader(void *userdata, const char *block, size_t length)
     if (length < 1) return 0;
     quota_context *ctx = (quota_context *) userdata;
 
-    char *quota = strndup(block, length);
+    char *quota = ne_strndup(block, length);
     if (!quota) {
         ctx->error = 1;
         return 0;
@@ -1865,17 +1959,6 @@ quota_reader(void *userdata, const char *block, size_t length)
 
 /* Reads available and used bytes from set and stores them in
    userdata. */
-#if NE_VERSION_MINOR < 26
-
-static void
-quota_result(void *userdata, const char *href, const ne_prop_result_set *set)
-{
-    quota_context *ctx = (quota_context *) userdata;
-    if (!ctx || !href || !set)
-        return;
-
-#else /* NE_VERSION_MINOR >= 26 */
-
 static void
 quota_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
 {
@@ -1883,8 +1966,6 @@ quota_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
     if (!ctx  || !uri || !uri->path || !set)
         return;
 
-#endif /* NE_VERSION_MINOR >= 26 */
-
     const char *data = ne_propset_value(set, &quota_names[AVAILABLE]);
     if (data) {
         ctx->total = strtoull(data, NULL, 10);
@@ -1921,6 +2002,14 @@ quota_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
 static int
 ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert)
 {
+    if (server_cert) {
+        if (ne_ssl_cert_cmp(cert, server_cert) == 0)
+            return 0;
+        if (have_terminal)
+            error(0, 0, _("the server certificate is not trusted"));
+        return -1;
+    }
+
     char *issuer = ne_ssl_readable_dname(ne_ssl_cert_issuer(cert));
     char *subject = ne_ssl_readable_dname(ne_ssl_cert_subject(cert));
     char *digest = ne_calloc(NE_SSL_DIGESTLEN);
@@ -2001,43 +2090,3 @@ ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert)
     return ret;
 }
 
-
-#if NE_VERSION_MINOR > 25
-static int
-update_cookie(ne_request *req, void *userdata, const ne_status *status)
-{
-    if (status->klass != 2)
-        return NE_OK;
-
-    const char *cookie_hdr = ne_get_response_header(req, "Set-Cookie2");
-    if (!cookie_hdr) {
-        cookie_hdr = ne_get_response_header(req, "Set-Cookie");
-    }
-    if (!cookie_hdr)
-        return NE_OK;
-
-    if (cookie && strstr(cookie_hdr, cookie) == cookie_hdr)
-        return NE_OK;
-
-    char *sep = strpbrk(cookie_hdr, "\",; \n\r\0");
-    if (!sep)
-        return NE_OK;
-    if (*sep == '\"')
-        sep = strpbrk(sep + 1, "\"");
-    if (!sep)
-        return NE_OK;
-
-    if (cookie) {
-        ne_unhook_pre_send(session, add_header, cookie);
-        free(cookie);
-        cookie = NULL;
-    }
-
-    char *value = ne_strndup(cookie_hdr, sep - cookie_hdr + 1);
-    cookie = ne_concat("Cookie: $Version=1;", value, "\r\n", NULL);
-    free(value);
-
-    ne_hook_pre_send(session, add_header, cookie);
-    return NE_OK;
-}
-#endif /* NE_VERSION_MINOR > 25 */
